// ==============================================================
// Vivado(TM) HLS - High-Level Synthesis from C, C++ and SystemC v2020.1.1 (64-bit)
// Copyright 1986-2020 Xilinx, Inc. All Rights Reserved.
// ==============================================================
/***************************** Include Files *********************************/
#include "xaccumulate_accel.h"

/************************** Function Implementation *************************/
#ifndef __linux__
int XAccumulate_accel_CfgInitialize(XAccumulate_accel *InstancePtr, XAccumulate_accel_Config *ConfigPtr) {
    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(ConfigPtr != NULL);

    InstancePtr->Control_BaseAddress = ConfigPtr->Control_BaseAddress;
    InstancePtr->IsReady = XIL_COMPONENT_IS_READY;

    return XST_SUCCESS;
}
#endif

void XAccumulate_accel_Start(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL) & 0x80;
    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL, Data | 0x01);
}

u32 XAccumulate_accel_IsDone(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL);
    return (Data >> 1) & 0x1;
}

u32 XAccumulate_accel_IsIdle(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL);
    return (Data >> 2) & 0x1;
}

u32 XAccumulate_accel_IsReady(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL);
    // check ap_start to see if the pcore is ready for next input
    return !(Data & 0x1);
}

void XAccumulate_accel_EnableAutoRestart(XAccumulate_accel *InstancePtr) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL, 0x80);
}

void XAccumulate_accel_DisableAutoRestart(XAccumulate_accel *InstancePtr) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_AP_CTRL, 0);
}

void XAccumulate_accel_Set_img_in1_V(XAccumulate_accel *InstancePtr, u32 Data) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_IN1_V_DATA, Data);
}

u32 XAccumulate_accel_Get_img_in1_V(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_IN1_V_DATA);
    return Data;
}

void XAccumulate_accel_Set_img_in2_V(XAccumulate_accel *InstancePtr, u32 Data) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_IN2_V_DATA, Data);
}

u32 XAccumulate_accel_Get_img_in2_V(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_IN2_V_DATA);
    return Data;
}

void XAccumulate_accel_Set_img_out_V(XAccumulate_accel *InstancePtr, u32 Data) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_OUT_V_DATA, Data);
}

u32 XAccumulate_accel_Get_img_out_V(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IMG_OUT_V_DATA);
    return Data;
}

void XAccumulate_accel_Set_height(XAccumulate_accel *InstancePtr, u32 Data) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_HEIGHT_DATA, Data);
}

u32 XAccumulate_accel_Get_height(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_HEIGHT_DATA);
    return Data;
}

void XAccumulate_accel_Set_width(XAccumulate_accel *InstancePtr, u32 Data) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_WIDTH_DATA, Data);
}

u32 XAccumulate_accel_Get_width(XAccumulate_accel *InstancePtr) {
    u32 Data;

    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Data = XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_WIDTH_DATA);
    return Data;
}

void XAccumulate_accel_InterruptGlobalEnable(XAccumulate_accel *InstancePtr) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_GIE, 1);
}

void XAccumulate_accel_InterruptGlobalDisable(XAccumulate_accel *InstancePtr) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_GIE, 0);
}

void XAccumulate_accel_InterruptEnable(XAccumulate_accel *InstancePtr, u32 Mask) {
    u32 Register;

    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Register =  XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IER);
    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IER, Register | Mask);
}

void XAccumulate_accel_InterruptDisable(XAccumulate_accel *InstancePtr, u32 Mask) {
    u32 Register;

    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    Register =  XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IER);
    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IER, Register & (~Mask));
}

void XAccumulate_accel_InterruptClear(XAccumulate_accel *InstancePtr, u32 Mask) {
    Xil_AssertVoid(InstancePtr != NULL);
    Xil_AssertVoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    XAccumulate_accel_WriteReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_ISR, Mask);
}

u32 XAccumulate_accel_InterruptGetEnabled(XAccumulate_accel *InstancePtr) {
    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    return XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_IER);
}

u32 XAccumulate_accel_InterruptGetStatus(XAccumulate_accel *InstancePtr) {
    Xil_AssertNonvoid(InstancePtr != NULL);
    Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

    return XAccumulate_accel_ReadReg(InstancePtr->Control_BaseAddress, XACCUMULATE_ACCEL_CONTROL_ADDR_ISR);
}

